;	This program is free software; you can redistribute it and/or modify
;	it under the terms of the GNU General Public License as published by
;	the Free Software Foundation; either version 2 of the License, or
;	(at your option) any later version.
;
;	This program is distributed in the hope that it will be useful,
;	but WITHOUT ANY WARRANTY; without even the implied warranty of
;	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;	GNU General Public License for more details.
;
;	You should have received a copy of the GNU General Public License
;	along with this program; if not, write to the Free Software
;	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
BCD_Multiply:
	;Inputs - HL and DE points to numbers, HL is multiplied by DE
	;Outputs - HL points to the result
	inc hl
	ld a,(hl)
	dec hl
	and 80h
	ld b,a
	inc de
	ld a,(de)
	dec de
	and 80h
	xor b
	ld (temp1),a		;Bit 7 of (temp1) is the sign of the result
	push hl
	push de
	ld c,(hl)
	inc hl
	ld b,(hl)
	res 7,b
	ld a,(de)
	ld l,a
	inc de
	ld a,(de)
	res 7,a
	ld h,a
	add hl,bc		;total length of the numbers
	push hl
	inc hl
	inc hl
	ld de,(areaFreeStart)
	ld b,d
	ld c,e
	add hl,de
	ld de,(areaFreeEnd)
	or a
	sbc hl,de
	jp nc,MemoryError
	add hl,de
	ld (areaFreeStart),hl
	;BC is where to start...
	ld h,b
	ld l,c
	pop bc
	;HL is the first byte of the result...BC is the number of bytes of it
	ld (temp7),hl
	ld (hl),c
	inc hl
	ld (hl),b
BCD_Multiply_ZeroLoop:
	inc hl
	ld (hl),0
	dec bc
	ld a,b
	or c
	jr nz,BCD_Multiply_ZeroLoop
	;HL is the rightmost byte of the result...
	ld (temp2),hl		;(temp2) stores where on the result to start in the loop
	pop hl		;original DE
	ld c,(hl)
	inc hl
	ld b,(hl)
	res 7,b
	add hl,bc		;HL is rightmost byte of DE
	ex de,hl			;DE is rightmost byte of the DE number, BC is length
	pop hl			;original HL
	push bc
	ld c,(hl)
	inc hl
	ld b,(hl)
	res 7,b
	add hl,bc		;HL is righmost byte of HL
	;DE is the rightmost byte of the DE number...
	ld (temp3),hl		;(temp3) is the starting right digit of the first number
	ld (temp4),bc		;(temp4) is the length of the first number
	pop bc
	;DE is the rightmost byte of the second number....BC is its length
	;In a loop, loop through the bytes of DE from right to left...
	;For each byte, loop through the first number from right to left, multiplying and
	;storing the bytes...
BCD_Multiply_Loop:
	bit onInterrupt,(iy+onFlags)
	jp nz,BreakError
	push bc
	;DE is the digit...
	ld ix,(temp2)
	ld hl,(temp3)
	ld bc,(temp4)
	xor a
	ld (temp5),a			;(temp5) is the carry to add to the result...
BCD_Multiply_InsideLoop:
	in a,(4)
	and 8h
	jp z,BreakError
	;Multiply the digit in (DE) by the digit in (HL)...then decrement HL, BC, move on
	;multiply here
	push bc
	push de
	push hl
	ld a,(de)
	ld b,(hl)		;A and B must be multiplied...we have table. ((a+b)^2)/4 - ((a-b)^2)/4
	push af
	push bc
	add a,b
	daa
	ld e,a
	ld a,0
	adc a,0
	ld d,a
	;DE is the (a+b)...in DAA format
	ex de,hl
	add hl,hl
	ld de,Multiply_Table
	add hl,de
	;HL points to where the first value is...
	ld e,(hl)
	inc hl
	ld d,(hl)
	;DE is the value...
	ld (temp6),de		;store first part in temp6
	pop bc
	pop af
	;A and B...must be compared
	cp b
	jr nc,BCD_Multiply_AMoreEqualB
	ld c,a
	ld a,b
	ld b,c
BCD_Multiply_AMoreEqualB
	sub b
	daa
	ld l,a
	ld h,0
	add hl,hl
	ld de,Multiply_Table
	add hl,de
	;HL is the second value...
	ld e,(hl)
	inc hl
	ld d,(hl)
	;DE is the second value
	ld hl,(temp6)
	;HL is first...subtract DE from HL
	ld a,l
	sub e
	daa
	ld l,a
	ld a,h
	sbc d
	daa
	ld h,a
	;HL now is the final, BCD answer...
	ld a,(temp5)		;carry from previous digit?
	add a,l
	daa		;least significant digit of answer plus carry...must add this to IX...the carry of that result is added to the higher byte of the multiplication result, which is put into the carry
	ld b,a
	ld a,h
	adc a,0
	ld h,a	;if the add carried, we had to increment the high byte of hte result...
	ld a,b
	add a,(ix)
	daa
	ld (ix),a
	ld a,h
	adc a,0
	ld (temp5),a 		;we have our next carry
	pop hl
	pop de
	pop bc
	;add result to IX, store, decrement IX
	dec ix
	dec hl
	dec bc
	ld a,b
	or c
	jr nz,BCD_Multiply_InsideLoop
	;Done...but what about any carry left?
	ld a,(temp5)
	add a,(ix)
	daa
	ld (ix),a

  	ld hl,(temp2)
	dec hl
	ld (temp2),hl
	dec de
	pop bc
	dec bc
	ld a,b
	or c
	jp nz,BCD_Multiply_Loop
	;Done
	ld hl,(temp7)
	;Remove superfluous zeros
	push hl
	ld ix,0
	ld c,(hl)
	inc hl
	ld b,(hl)
	res 7,b
	inc hl
BCD_Multiply_Clean:
	ld a,(hl)
	or a
	jr nz,BCD_Multiply_GoClean
	inc hl
	inc ix
	dec bc
	ld a,b
	or c
	jr nz,BCD_Multiply_Clean
	;The whole number was zeros...we only need one of them
	pop hl
	ld c,(hl)
	ld (hl),1
	inc hl
	ld b,(hl)
	res 7,b
	ld a,1000000b
	and (hl)
	ld (hl),a
	;BC is size it was...
	dec bc
	ld hl,(areaFreeStart)
	or a
	sbc hl,bc
	ld (areaFreeStart),hl
	jr BCD_Multiply_SignFix
BCD_Multiply_GoClean:
	;BC is the size that was nonzero
	pop de
	ld a,c
	ld (de),a
	inc de
	ld a,b
	ld (de),a
	inc de
	push bc
	ldir		;shift down...
	pop bc
	ld hl,(areaFreeStart)
	push ix
	pop de
	or a
	sbc hl,de
	ld (areaFreeStart),hl
BCD_Multiply_SignFix:
	ld a,(temp1)
	ld hl,(temp7)
	inc hl
	res 7,(hl)
	xor (hl)
	ld (hl),a
	dec hl
	ret